#include "adhydro.h"
#include "file_manager.h"

void ADHydro::setLoadBalancingToManual()
{
  // Charm++ call to set load balancing to manual.
  TurnManualLBOn();
}

ADHydro::ADHydro(CkArgMsg* msg)
{
  // Create file manager.
  fileManagerProxy = CProxy_FileManager::ckNew();
  
  // Set the callback to continue initialization when the file manager is ready.
  fileManagerProxy.ckSetReductionClient(new CkCallback(CkReductionTarget(ADHydro, fileManagerInitialized), thisProxy));
  
  // Initialize the file manager.
  fileManagerProxy.initializeFromNetCDFFiles(strlen(INPUT_DIRECTORY) + 1, INPUT_DIRECTORY);
}

void ADHydro::fileManagerInitialized()
{
  // Initialize member variables.
  currentTime             = fileManagerProxy.ckLocalBranch()->currentTime;
  endTime                 = currentTime + 12000.0; // FIXME make command line parameter
  dt                      = fileManagerProxy.ckLocalBranch()->dt;
  outputPeriod            = 3600.0; // FIXME make command line parameter
  nextOutputTime          = currentTime + outputPeriod;
  iteration               = fileManagerProxy.ckLocalBranch()->iteration;
  writeGeometry           = true;
  writeParameter          = true;
  needToUpdateForcingData = true;
  needToCheckInvariant    = true;
  
  // Create mesh and channels.
  if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfMeshElements)
    {
      meshProxy = CProxy_MeshElement::ckNew(fileManagerProxy.ckLocalBranch()->globalNumberOfMeshElements);
    }
  
  if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfChannelElements)
    {
      channelProxy = CProxy_ChannelElement::ckNew(fileManagerProxy.ckLocalBranch()->globalNumberOfChannelElements);
    }

  // Initialize mesh and channels.
  if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfMeshElements)
    {
      meshProxy.initialize(channelProxy, fileManagerProxy);
    }
  
  if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfChannelElements)
    {
      channelProxy.initialize(meshProxy, fileManagerProxy);
    }
  
  checkForcingData();
}

void ADHydro::checkForcingData()
{
      // Set callback.
      if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfMeshElements)
        {
          meshProxy.ckSetReductionClient(new CkCallback(CkReductionTarget(ADHydro, meshForcingDataDone), thisProxy));
        }
      else
        {
          thisProxy.meshForcingDataDone();
        }
      
      if (0 < fileManagerProxy.ckLocalBranch()->globalNumberOfChannelElements)
        {
          channelProxy.ckSetReductionClient(new CkCallback(CkReductionTarget(ADHydro, channelForcingDataDone), thisProxy));
        }
      else
        {
          thisProxy.channelForcingDataDone();
        }

      // Update forcing data.
      fileManagerProxy.readForcingData(meshProxy, channelProxy, currentTime, strlen(INPUT_DIRECTORY) + 1, INPUT_DIRECTORY);
      thisProxy.waitForForcingDataToFinish();
}

// Suppress warnings in the The Charm++ autogenerated code.
#pragma GCC diagnostic ignored "-Wunused-variable"
#include "adhydro.def.h"
#pragma GCC diagnostic warning "-Wunused-variable"
