#include "file_manager.h"
#include "all.h"
#include <netcdf.h>
//#include <netcdf_par.h>

FileManager::FileManager()
{
  globalNumberOfMeshNodes        = 0;
  localMeshNodeStart             = 0;
  localNumberOfMeshNodes         = 0;
  globalNumberOfMeshElements     = 0;
  localMeshElementStart          = 0;
  localNumberOfMeshElements      = 0;
  globalNumberOfChannelNodes     = 0;
  localChannelNodeStart          = 0;
  localNumberOfChannelNodes      = 0;
  globalNumberOfChannelElements  = 0;
  localChannelElementStart       = 0;
  localNumberOfChannelElements   = 0;
  meshMeshNeighbors              = NULL;
  meshChannelNeighbors           = NULL;
  channelChannelNeighbors        = NULL;
  channelMeshNeighbors           = NULL;
  currentTime                    = 0.0;
  dt                             = 1.0;
  iteration                      = 1;
}

void FileManager::readForcingData(CProxy_MeshElement meshProxy, CProxy_ChannelElement channelProxy, double currentTime, size_t directorySize,
                                  const char* directory)
{
  meshProxy.forcingDataMessage(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
  channelProxy.forcingDataMessage();
}

void FileManager::initializeFromNetCDFFiles(size_t directorySize, const char* directory)
{
  int    fileID;                 // ID of NetCDF file.
  int    varID;                  // ID of variable in NetCDF file.
  size_t start[NC_MAX_VAR_DIMS]; // For specifying subarrays when writing to NetCDF file.
  size_t count[NC_MAX_VAR_DIMS]; // For specifying subarrays when writing to NetCDF file.
  
  // Set variables.
  iteration = 1;
  currentTime = 0.0;
  dt = 1.0;
  globalNumberOfMeshNodes = 19337;
  globalNumberOfMeshElements = 36861;
  globalNumberOfChannelNodes = 165343;
  globalNumberOfChannelElements = 14974;
  localMeshNodeStart = 0;
  localMeshElementStart = 0;
  localChannelNodeStart = 0;
  localChannelElementStart = 0;
  localNumberOfMeshNodes = globalNumberOfMeshNodes;
  localNumberOfMeshElements = globalNumberOfMeshElements;
  localNumberOfChannelNodes = globalNumberOfChannelNodes;
  localNumberOfChannelElements = globalNumberOfChannelElements;
  
  // Open file.
  nc_open(INPUT_DIRECTORY"/geometry.nc", NC_NOWRITE, &fileID);
  //nc_open_par(INPUT_DIRECTORY"/geometry.nc", NC_NETCDF4 | NC_MPIIO, MPI_COMM_WORLD, MPI_INFO_NULL, &fileID);
  
  nc_inq_varid(fileID, "meshMeshNeighbors", &varID);
  meshMeshNeighbors = new int[localNumberOfMeshElements][MeshElement::meshNeighborsSize];
  start[0]          = 0;
  start[1]          = localMeshElementStart;
  start[2]          = 0;
  count[0]          = 1;
  count[1]          = localNumberOfMeshElements;
  count[2]          = MeshElement::meshNeighborsSize;
  nc_get_vara_int(fileID, varID, start, count, (int*)meshMeshNeighbors);
  
  nc_inq_varid(fileID, "meshChannelNeighbors", &varID);
  meshChannelNeighbors = new int[localNumberOfMeshElements][MeshElement::channelNeighborsSize];
  start[0]             = 0;
  start[1]             = localMeshElementStart;
  start[2]             = 0;
  count[0]             = 1;
  count[1]             = localNumberOfMeshElements;
  count[2]             = MeshElement::channelNeighborsSize;
  nc_get_vara_int(fileID, varID, start, count, (int*)meshChannelNeighbors);
  
  nc_inq_varid(fileID, "channelChannelNeighbors", &varID);
  channelChannelNeighbors = new int[localNumberOfChannelElements][ChannelElement::channelNeighborsSize];
  start[0]                = 0;
  start[1]                = localChannelElementStart;
  start[2]                = 0;
  count[0]                = 1;
  count[1]                = localNumberOfChannelElements;
  count[2]                = ChannelElement::channelNeighborsSize;
  nc_get_vara_int(fileID, varID, start, count, (int*)channelChannelNeighbors);

  nc_inq_varid(fileID, "channelMeshNeighbors", &varID);
  channelMeshNeighbors = new int[localNumberOfChannelElements][ChannelElement::meshNeighborsSize];
  start[0]             = 0;
  start[1]             = localChannelElementStart;
  start[2]             = 0;
  count[0]             = 1;
  count[1]             = localNumberOfChannelElements;
  count[2]             = ChannelElement::meshNeighborsSize;
  nc_get_vara_int(fileID, varID, start, count, (int*)channelMeshNeighbors);

  nc_close(fileID);
  
  contribute();
}

// Suppress warnings in the The Charm++ autogenerated code.
#pragma GCC diagnostic ignored "-Wunused-variable"
#pragma GCC diagnostic ignored "-Wsign-compare"
#include "file_manager.def.h"
#pragma GCC diagnostic warning "-Wsign-compare"
#pragma GCC diagnostic warning "-Wunused-variable"
