#ifndef __FILE_MANAGER_H__
#define __FILE_MANAGER_H__

// file_manager.decl.h needs CProxy_MeshElement from mesh_element.decl.h and CProxy_ChannelElement from channel_element.decl.h.
// mesh_element.decl.h needs CProxy_FileManager from file_manager.decl.h and CProxy_ChannelElement from channel_element.decl.h.
// channel_element.decl.h needs CProxy_FileManager from file_manager.decl.h and CProxy_MeshElement from mesh_element.decl.h.
// These forward declarations break the circularity.
class CProxy_MeshElement;
class CProxy_ChannelElement;

// Suppress warnings in the The Charm++ autogenerated code.
#pragma GCC diagnostic ignored "-Wsign-compare"
#include "file_manager.decl.h"
#pragma GCC diagnostic warning "-Wsign-compare"

#include "mesh_element.h"
#include "channel_element.h"

typedef int    intarraymmn[MeshElement::meshNeighborsSize];           // Fixed size array of ints.     Size is mesh    mesh neighbors.
typedef int    intarraymcn[MeshElement::channelNeighborsSize];        // Fixed size array of ints.     Size is mesh    channel neighbors.
typedef int    intarrayccn[ChannelElement::channelNeighborsSize];     // Fixed size array of ints.     Size is channel channel neighbors.
typedef int    intarraycmn[ChannelElement::meshNeighborsSize];        // Fixed size array of ints.     Size is channel mesh neighbors.

// The group of file managers acts as an in-memory cache for values that are
// read from and written to NetCDF files.  Reading and writing individual
// values from NetCDF files is too slow.  We need to read and write large
// blocks from the arrays in the NetCDF files.  However, individual values are
// needed by mesh and channel elements.  The file manager group does the block
// reads and writes and makes the values available to the elements.
//
// At initialization, each file manager takes ownership of a block of each
// array containing the data for the elements that get assigned to its
// processor by the default Charm++ block mapping of array elements to
// processors.  This way, elements can initialize themselves without any
// message passing by getting a pointer to the file manager local branch and
// accessing its public member variables.
//
// After initialization, array elements might migrate away from the file
// manager local branch that owns their data.  We do not transfer ownership of
// the data between file manager local branches.  Instead, message passing is
// used to update data after initialization.
class FileManager : public CBase_FileManager
{
public:
  // Constructor.  Initializes the file manager to hold no data.
  FileManager();
  
  // Read forcing data from file and send to mesh elements.  Each file manager
  // reads the data for the mesh elements it owns and sends it on to those mesh
  // elements.
  //
  // Parameters:
  //
  // meshProxy     - The mesh elements to send the forcing data to.
  // channelProxy  - The channel elements to send the forcing data to.
  // currentTime   - The updated time step in seconds.
  // directorySize - The size of the array passed in to directory.
  // directory     - The directory from which to read the files.
  void readForcingData(CProxy_MeshElement meshProxy, CProxy_ChannelElement channelProxy, double currentTime, size_t directorySize, const char* directory);
  
  // Initializes the file manager from NetCDF files.  When done the file
  // managers contribute to an empty reduction.
  //
  // Parameters:
  //
  // directorySize - The size of the array passed in to directory.
  // directory     - The directory from which to read the files.
  void initializeFromNetCDFFiles(size_t directorySize, const char* directory);
  
  int globalNumberOfMeshNodes;       // Number of mesh nodes across all file managers.
  int localMeshNodeStart;            // Index of first mesh node owned by this local branch.
  int localNumberOfMeshNodes;        // Number of mesh nodes owned by this local branch.
  int globalNumberOfMeshElements;    // Number of mesh elements across all file managers.
  int localMeshElementStart;         // Index of first mesh element owned by this local branch.
  int localNumberOfMeshElements;     // Number of mesh elements owned by this local branch.
  int globalNumberOfChannelNodes;    // Number of channel nodes across all file managers.
  int localChannelNodeStart;         // Index of first channel node owned by this local branch.
  int localNumberOfChannelNodes;     // Number of channel nodes owned by this local branch.
  int globalNumberOfChannelElements; // Number of channel elements across all file managers.
  int localChannelElementStart;      // Index of first channel element owned by this local branch.
  int localNumberOfChannelElements;  // Number of channel elements owned by this local branch.
  
  intarraymmn*    meshMeshNeighbors;
  intarraymcn*    meshChannelNeighbors;
  intarrayccn*     channelChannelNeighbors;
  intarraycmn*     channelMeshNeighbors;
  
  // Time information.  This is only kept up to date when inputting or outputting.
  double          currentTime; // Seconds.
  double          dt;          // Next timestep duration in seconds.
  CMK_REFNUM_TYPE iteration;   // Iteration number to put on all messages this timestep.
};

#endif // __FILE_MANAGER_H__
